@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (C) 2010-2018 by Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
+# Copyright (C) 2010-2020 by Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
 #
 # Python X2Go is free software; you can redistribute it and/or modify
 # it under the terms of the GNU Affero General Public License as published by
@@ -1771,7 +1771,8 @@ class X2GoControlSession(paramiko.SSHClient):
                     _stdout = stdout.read()
                     if sys.version_info[0] >= 3:
                         _stdout = _stdout.decode()
-                    _listsessions = self._list_backend(_stdout, info_backend=self._info_backend).sessions
+                    _l = self._list_backend(_stdout, info_backend=self._info_backend)
+                    _listsessions = _l.get_sessions()
                     _success = True
                 except KeyError:
                     gevent.sleep(1)
