@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (C) 2010-2018 by Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
+# Copyright (C) 2010-2020 by Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
 #
 # Python X2Go is free software; you can redistribute it and/or modify
 # it under the terms of the GNU Affero General Public License as published by
@@ -42,10 +42,15 @@ class X2GoServerSessionInfo(object):
     that is retrieved from the connected X2Go server on
     ``X2GoTerminalSession.start()`` resp. ``X2GoTerminalSession.resume()``.
 
-
     """
+    def __init__(self):
+        self.initialized = False
+        self.protected = False
+        self.session_type_cached = None
+
     def __str__(self):
         return self.name
+
     def __repr__(self):
         result = 'X2GoServerSessionInfo('
         for p in dir(self):
@@ -78,6 +83,7 @@ class X2GoServerSessionInfo(object):
             self.username = l[11]
             self.sshfs_port = int(l[13])
             self.local_container = ''
+            self.initialized = True
         except IndexError as e:
             # DEBUGGING CODE
             raise e
@@ -102,10 +108,13 @@ class X2GoServerSessionInfo(object):
 
         :returns: returns ``True`` if this session is a published applications provider
 
-        :rtype: ``bool``
+        :rtype: ``bool`` or ``None`` (if not initialized)
 
         """
-        return bool(re.match('.*_stRPUBLISHED_.*', self.name))
+        if self.initialized:
+            return bool(re.match('.*_stRPUBLISHED_.*', self.name))
+        else:
+            return None
 
     def is_running(self):
         """\
@@ -114,27 +123,35 @@ class X2GoServerSessionInfo(object):
 
         :returns: ``True`` if the session is running, ``False`` otherwise
 
-        :rtype: ``bool``
+        :rtype: ``bool`` or ``None`` (if not initialized)
 
         """
-        return self.status == 'R'
+        if self.initialized:
+            return self.status == 'R'
+        else:
+            return None
 
     def get_session_type(self):
         """\
-        Get the session type (i.e. 'D', 'R', 'S' or 'P').
+        Get the session type (i.e. 'D', 'K', 'R', 'S' or 'P').
 
 
         :returns: session type
 
-        :rtype: ``str``
+        :rtype: ``str`` or ``None`` (if not initialized)
 
         """
-        cmd = self.name.split('_')[1]
-        print(cmd)
-        session_type = cmd[2]
-        if session_type == 'R' and self.is_published_applications_provider():
-            session_type = 'P'
-        return session_type
+        if self.session_type_cached is not None:
+            return self.session_type_cached
+        if self.initialized:
+            cmd = self.name.split('_')[1]
+            session_type = cmd[2]
+            if session_type == 'R' and self.is_published_applications_provider():
+                session_type = 'P'
+            self.session_type_cached = session_type
+            return session_type
+        else:
+            return None
 
     def get_share_mode(self):
         """\
@@ -143,15 +160,18 @@ class X2GoServerSessionInfo(object):
 
         :returns: share mode (0: view-only, 1: full access), ``None`` when used for non-desktop-sharing sessions
 
-        :rtype: ``str``
+        :rtype: ``str`` or ``None`` (if not initialized)
 
         """
-        share_mode = None
-        cmd = self.name.split('_')[1]
-        session_type = cmd[2]
-        if session_type == 'S':
-            share_mode = cmd[3]
-        return share_mode
+        if self.initialized:
+            share_mode = None
+            cmd = self.name.split('_')[1]
+            session_type = cmd[2]
+            if session_type == 'S':
+                share_mode = cmd[3]
+            return share_mode
+        else:
+            return None
 
     def is_suspended(self):
         """\
@@ -160,10 +180,13 @@ class X2GoServerSessionInfo(object):
 
         :returns: ``True`` if the session is suspended, ``False`` otherwise
 
-        :rtype: ``bool``
+        :rtype: ``bool`` or ``None`` (if not initialized)
 
         """
-        return self.status == 'S'
+        if self.initialized:
+            return self.status == 'S'
+        else:
+            return None
 
     def is_desktop_session(self):
         """\
@@ -172,10 +195,67 @@ class X2GoServerSessionInfo(object):
 
         :returns: ``True`` if this session is a desktop session, ``False`` otherwise
 
+        :rtype: ``bool`` or ``None`` (if not initialized)
+
+        """
+        if self.initialized:
+            return self.get_session_type() in ('D', 'K')
+        else:
+            return None
+
+    def is_kdrive_session(self):
+        """\
+        Is this session a KDrive based desktop session?
+
+
+        :returns: ``True`` if this session is a KDrive based desktop session, ``False`` otherwise
+
         :rtype: ``bool``
 
         """
-        return self.get_session_type() == 'D'
+        if self.initialized:
+            return self.get_session_type() == 'K'
+        else:
+           return None
+
+    def is_nx3_session(self):
+        """\
+        Is this session an NXv3 based desktop/rootless/shadow session?
+
+
+        :returns: ``True`` if this session is a NXv3 based desktop, rootless or shadow session, ``False`` otherwise
+
+        :rtype: ``bool``
+
+        """
+        if self.initialized:
+            return self.get_session_type() in ('D', 'S', 'R', 'P')
+        else:
+            return None
+
+    def is_kdrive_session(self):
+        """\
+        Is this session a KDrive based desktop session?
+
+
+        :returns: ``True`` if this session is a KDrive based desktop session, ``False`` otherwise
+
+        :rtype: ``bool``
+
+        """
+        return self.get_session_type() == 'K'
+
+    def is_nx3_session(self):
+        """\
+        Is this session an NXv3 based desktop/rootless session?
+
+
+        :returns: ``True`` if this session is a NXv3 based desktop/rootless session, ``False`` otherwise
+
+        :rtype: ``bool``
+
+        """
+        return self.get_session_type() in ('D','R','P')
 
     def _parse_x2gostartagent_output(self, x2go_output):
         """\
@@ -203,6 +283,7 @@ class X2GoServerSessionInfo(object):
             self.status = 'R'
             self.local_container = ''
             self.remote_container = ''
+            self.initialized = True
         except IndexError as e:
             # DEBUGGING CODE
             raise e
@@ -243,6 +324,7 @@ class X2GoServerSessionInfo(object):
         self.hostname = hostname
         self.local_container = local_container
         self.remote_container = remote_container
+        self.initialized = True
 
     def protect(self):
         """\
@@ -268,6 +350,14 @@ class X2GoServerSessionInfo(object):
         """
         return self.protected
 
+    def is_initialized(self):
+        """\
+        Check if this session info data structure has been initialized.
+
+
+        """
+        return self.initialized
+
     def get_status(self):
         """\
         Retrieve the session's status from this session info data structure.
@@ -275,10 +365,13 @@ class X2GoServerSessionInfo(object):
 
         :returns: session status
 
-        :rtype: ``str``
+        :rtype: ``str`` or ``None`` (if not initalized)
 
         """
-        return self.status
+        if self.initialized:
+            return self.status
+        else:
+            return None
 
     def clear(self):
         """\
@@ -303,6 +396,8 @@ class X2GoServerSessionInfo(object):
         self.local_container = ''
         self.remote_container = ''
         self.protected = False
+        self.initialized = False
+        self.session_type_cached = None
 
     def update(self, session_info):
         """\
@@ -359,6 +454,16 @@ class X2GoServerSessionList(object):
     def __call__(self):
         return self.sessions
 
+    def get_sessions(self):
+        """\
+        Get the complete sessions property.
+
+        :return: get this instance's dictionary of sessions
+        :rtype: ``dict``
+
+        """
+        return self.sessions
+
     def set_sessions(self, sessions):
         """\
         Set the sessions property directly by parsing a complete data structure.
