@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (C) 2010-2018 by Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
+# Copyright (C) 2010-2020 by Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
 #
 # Python X2Go is free software; you can redistribute it and/or modify
 # it under the terms of the GNU Affero General Public License as published by
@@ -34,6 +34,7 @@ __name__    = 'x2go.backends.printing.file'
 
 # modules
 import types
+import sys
 
 # Python X2Go modules
 import x2go.log as log
@@ -205,8 +206,9 @@ class X2GoClientPrinting(inifiles.X2GoIniFile):
             _ini_section = _print_property_map[print_property]['ini_section']
             _ini_option = _print_property_map[print_property]['ini_option']
             _default_type = self.get_type(_ini_section, _ini_option)
-            if type(value) is str:
-                value = value.encode('utf-8')
+            if sys.version_info[0] < 3:
+                if type(value) is str:
+                    value = value.encode('utf-8')
             if  _default_type != type(value):
                 raise x2go_exceptions.X2GoClientPrintingException('Type mismatch error for property ,,%s\'\' - is: %s, should be: %s' % (print_property, str(type(value)), str(_default_type)))
             self.update_value(_ini_section, _ini_option, value)
