@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (C) 2010-2018 by Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
+# Copyright (C) 2010-2020 by Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
 #
 # Python X2Go is free software; you can redistribute it and/or modify
 # it under the terms of the GNU Affero General Public License as published by
@@ -18,7 +18,7 @@
 # 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
 
 """\
-X2GoProxy classes - proxying your connection through NX3 and others.
+X2GoProxy class - proxying your graphical connection through NX3.
 
 """
 __NAME__ = 'x2goproxynx3-pylib'
@@ -122,7 +122,7 @@ class X2GoProxy(base.X2GoProxy):
             options = open(_options_filename, 'w')
             options.write(u'%s:%s' % (','.join(self.PROXY_OPTIONS), self.PROXY_DISPLAY))
             options.close()
-            self.PROXY_OPTIONS= [ 'nx/nx', 'options=%s' % os.path.join("\\", "..", "S-%s" % self.session_info.name, 'options'), ]
+            self.PROXY_OPTIONS= [ 'nx/nx', 'options=%s' % os.path.join(".", "..", "S-%s" % self.session_info.name, 'options'), ]
 
         cmd_line = [ self.PROXY_CMD, ]
         cmd_line.append(self.PROXY_MODE)
