@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (C) 2010-2018 by Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
+# Copyright (C) 2010-2020 by Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
 #
 # Python X2Go is free software; you can redistribute it and/or modify
 # it under the terms of the GNU Affero General Public License as published by
@@ -58,7 +58,8 @@ from x2go.defaults import X2GO_CLIENT_ROOTDIR as _X2GO_CLIENT_ROOTDIR
 from x2go.defaults import X2GO_SESSIONS_ROOTDIR as _X2GO_SESSIONS_ROOTDIR
 from x2go.defaults import X2GO_GENERIC_APPLICATIONS as _X2GO_GENERIC_APPLICATIONS
 from x2go.defaults import X2GO_DESKTOPSESSIONS as _X2GO_DESKTOPSESSIONS
-
+from x2go.defaults import X2GO_DESKTOPSESSIONS_NX3_PREFERRED as _X2GO_DESKTOPSESSIONS_NX3_PREFERRED
+from x2go.defaults import X2GO_DESKTOPSESSIONS_KDRIVE_PREFERRED as _X2GO_DESKTOPSESSIONS_KDRIVE_PREFERRED
 from x2go.defaults import BACKENDS as _BACKENDS
 
 _local_color_depth = utils.local_color_depth()
@@ -128,11 +129,12 @@ class X2GoSessionParams(object):
     def rewrite_session_type(self):
         """\
         Rewrite the X2Go session type, so that the X2Go server
-        can understand it (``desktop`` -> ``D``, etc.).
+        can understand it (``desktop`` -> ``D`` or ``K``, depending
+        on the proxy backend, etc.).
 
         Also if the object's ``command`` property is a known window
         manager, the session type will be set to 'D'
-        (i.e. desktop).
+        (i.e. desktop with NX3 backend).
 
         :returns: D' if session should probably a desktop session,
             'R' for rootless sessions, 'P' for sessions providing published
@@ -152,14 +154,16 @@ class X2GoSessionParams(object):
                 if self.geometry == 'fullscreen': self.session_type = 'D'
                 else: self.session_type = 'R'
             elif cmd == 'XDMCP':
-                self.session_type = 'D'
+                self.session_type = 'desktop'
             elif cmd in list(_X2GO_DESKTOPSESSIONS.keys()):
-                self.session_type = 'D'
+                self.session_type = 'desktop'
             elif os.path.basename(cmd) in list(_X2GO_DESKTOPSESSIONS.values()):
-                self.session_type = 'D'
+                self.session_type = 'desktop'
 
-        if self.session_type in ("D", "desktop"):
+        if self.session_type == "D" or ( self.session_type == "desktop" and self.kdrive == False ):
             self.session_type = 'D'
+        elif self.session_type == "K" or ( self.session_type == "desktop" and self.kdrive == True ):
+            self.session_type = 'K'
         elif self.session_type in ("S", "shared", "shadow"):
             self.session_type = 'S'
         elif self.session_type in ("R", "rootless", "application"):
@@ -213,6 +217,7 @@ class X2GoTerminalSession(object):
                  kbtype='null/null', kblayout='null', kbvariant='null',
                  clipboard='both',
                  xinerama=False,
+                 kdrive=False,
                  session_type="application", snd_system='pulse', snd_port=4713, cmd=None,
                  published_applications=False,
                  set_session_title=False, session_title="", applications=[],
@@ -258,10 +263,11 @@ class X2GoTerminalSession(object):
         :type kbvariant: ``str``
         :param clipboard: clipboard mode (``both``: bidirectional copy+paste, ``server``: copy+paste from server to
             client, ``client``: copy+paste from client to server, ``none``: disable clipboard completely
+        :type clipboard: ``str``
         :param xinerama: enable/disable Xinerama support in remote X2Go session
         :type xinerama: ``bool``
-        :type clipboard: ``str``
-        :param session_type: either ``desktop``, ``application`` (rootless session) or ``shared``
+        :param session_type: either ``desktop``, ``application`` (rootless session) or ``shared`` (only set for new sessions,
+            ignored for to-be-resumed sessions)
         :type session_type: ``str``
         :param snd_system: sound system to be used on server (``none``, ``pulse`` (default),
             ``arts`` (obsolete) or ``esd``)
@@ -379,13 +385,29 @@ class X2GoTerminalSession(object):
         self.params.server_encoding = str(server_encoding)
 
         self.params.rootdir = (type(rootdir) is bytes) and rootdir or self.sessions_rootdir
+
+        self.params.kdrive = False
+        # enforce X2GoKDrive usage?
+        if kdrive:
+            proxy_backend = "KDRIVE"
+            self.params.kdrive = True
+
         self.params.update()
 
         self.profile_name = profile_name
         self.set_session_title = set_session_title
         self.session_title = session_title
         self.session_window = None
-        self.proxy_backend = utils._get_backend_class(proxy_backend, "X2GoProxy")
+
+        # auto-detect graphical proxy/kdrive backend?
+        if proxy_backend == 'auto-detect':
+            # we need to defer this at this point
+            # until we know the exact session command
+            # to be launched... (i.e. what desktop environment
+            # the user wants to launch via X2Go
+            self.proxy_backend = proxy_backend
+        else:
+            self.proxy_backend = utils._get_backend_class(proxy_backend, "X2GoProxy")
 
         self.snd_port = snd_port
         self.print_action = print_action
@@ -431,15 +453,22 @@ class X2GoTerminalSession(object):
 
 
         """
-        if self._share_local_folder_lock.locked():
-            self._share_local_folder_lock.release()
+        try:
+            if self._share_local_folder_lock and self._share_local_folder_lock.locked():
+                self._share_local_folder_lock.release()
+        except AttributeError:
+            pass
+
         self.release_telekinesis()
         self.release_proxy()
         self.session_window = None
-        self.update_session_window_file()
 
         try:
+            self.update_session_window_file()
+        except AttributeError:
+            pass
 
+        try:
             if self.control_session.get_transport() is not None:
                 try:
                     for _tunnel in [ _tun[1] for _tun in list(self.reverse_tunnels[self.session_info.name].values()) ]:
@@ -457,7 +486,10 @@ class X2GoTerminalSession(object):
         except AttributeError:
             pass
 
-        self.session_info.clear()
+        try:
+            self.session_info.clear()
+        except AttributeError:
+            pass
 
     def _mk_sessions_rootdir(self, rootdir):
         """\
@@ -532,7 +564,9 @@ class X2GoTerminalSession(object):
 
     def get_session_type(self):
         """\
-        Retrieve the X2Go session's session type as stored in the session parameter object.
+        Retrieve the X2Go session's session type as stored either in the parameter
+        object (for sessions not yet launched) or in the ``session_info`` object (for
+        already launched / to-be-resumed sessions).
 
 
         :returns: the session type
@@ -540,7 +574,10 @@ class X2GoTerminalSession(object):
         :rtype: ``str``
 
         """
-        return self.params.session_type
+        if self.session_info.is_initialized():
+            return self.session_info.get_session_type()
+        else:
+            return self.params.session_type
 
     def start_sound(self):
         """\
@@ -1080,7 +1117,7 @@ class X2GoTerminalSession(object):
         # no blanks at beginning or end, no blanks-only...
         self.session_title = self.session_title.strip()
 
-        if self.params.session_type == 'D':
+        if self.get_session_type() == 'D':
             if self.set_session_title:
 
                 if not self.session_title:
@@ -1090,7 +1127,7 @@ class X2GoTerminalSession(object):
                 # session title fallback... (like X2Go server does it...)
                 self.session_title = _generic_title
 
-        elif self.params.session_type == 'S':
+        elif self.get_session_type() == 'S':
             if self.set_session_title:
 
                 shared_user = _generic_title.split('XSHAD')[1]
@@ -1345,7 +1382,7 @@ class X2GoTerminalSession(object):
                      str(self.session_info.snd_port),
                      _rewrite_blanks(_rewrite_cmd(cmd, params=self.params)),
                      str(self.params.snd_system),
-                     str(self.params.session_type),
+                     str(self.get_session_type()),
                      "1>/dev/null 2>/dev/null & exit",
                    ]
 
@@ -1423,7 +1460,7 @@ class X2GoTerminalSession(object):
         (stdin, stdout, stderr) = self.control_session._x2go_exec_command(cmd_line)
         _stderr = stderr.read()
         if sys.version_info[0] >= 3:
-            _stderr.decode()
+            _stderr = _stderr.decode()
         if not _stderr:
             self.logger('setting keyboard layout ,,%s\'\' and variant ,,%s\'\' for session %s has been successful' % (layout, variant, self.session_info), log.loglevel_NOTICE)
             return True
@@ -1546,6 +1583,27 @@ class X2GoTerminalSession(object):
 
         self.params.rewrite_session_type()
 
+        # auto-detect proxy backend, if requested
+        if self.proxy_backend == 'auto-detect':
+            if self.params.cmd in list(_X2GO_DESKTOPSESSIONS_KDRIVE_PREFERRED.keys()):
+                self.proxy_backend = utils._get_backend_class("KDRIVE", "X2GoProxy")
+                self.params.kdrive = True
+            elif os.path.basename(self.params.cmd) in list(_X2GO_DESKTOPSESSIONS_KDRIVE_PREFERRED.values()):
+                self.proxy_backend = utils._get_backend_class("KDRIVE", "X2GoProxy")
+                self.params.kdrive = True
+            elif self.params.cmd in list(_X2GO_DESKTOPSESSIONS_NX3_PREFERRED.keys()):
+                self.proxy_backend = utils._get_backend_class("NX3", "X2GoProxy")
+            elif os.path.basename(self.params.cmd) in list(_X2GO_DESKTOPSESSIONS_NX3_PREFERRED.values()):
+                self.proxy_backend = utils._get_backend_class("NX3", "X2GoProxy")
+            else:
+                self.proxy_backend = utils._get_backend_class("default", "X2GoProxy")
+
+        if self.params.kdrive and not 'X2GOKDRIVE_BASE_SUPPORT' in self.control_session.get_server_features():
+            self.logger('X2Go KDrive is not supported on the remote X2Go Server', loglevel=log.loglevel_ERROR)
+            raise x2go_exceptions.X2GoTerminalSessionException('X2Go KDrive is not support server-side')
+
+        self.params.rewrite_session_type()
+
         if self.params.geometry == 'maximize':
             _geometry = utils.get_workarea_geometry()
             if _geometry is None or len(_geometry) != 2:
@@ -1564,10 +1622,10 @@ class X2GoTerminalSession(object):
                      str(self.params.kblayout),
                      str(self.params.kbtype),
                      str(setkbd),
-                     str(self.params.session_type),
+                     str(self.get_session_type()),
                      str(self.params.cmd),
                    ]
-        if self.params.session_type != 'S':
+        if self.get_session_type() != 'S':
             cmd_line.append(
                      str(self.params.clipboard),
             )
@@ -1614,7 +1672,7 @@ class X2GoTerminalSession(object):
                                                                 self.session_info.name,
                                                                )
 
-        # set up SSH tunnel for X11 graphical elements
+        # set up SSH tunnel for X11 graphical elements and launch client-side proxy/kdrive client
         self.proxy = self.proxy_backend(session_info=self.session_info,
                                         ssh_transport=self.control_session.get_transport(),
                                         sessions_rootdir=self.sessions_rootdir,
@@ -1626,7 +1684,7 @@ class X2GoTerminalSession(object):
         if proxy_ok:
             self.active_threads.append(self.proxy)
 
-            if self.params.session_type in ('D', 'S'):
+            if self.get_session_type() in ('D', 'S', 'K'):
                 self.find_session_window()
                 self.auto_session_window_title()
                 self.raise_session_window()
@@ -1713,12 +1771,19 @@ class X2GoTerminalSession(object):
         self.session_info.remote_container = '%s/.x2go/C-%s' % (self.control_session._x2go_remote_home,
                                                                 self.session_info.name,
                                                                )
+
+        # make sure to use the correct proxy backend
+        if self.is_kdrive_session():
+            self.proxy_backend = utils._get_backend_class("KDRIVE", "X2GoProxy")
+        else:
+            self.proxy_backend = utils._get_backend_class("NX3", "X2GoProxy")
+
         self.proxy = self.proxy_backend(session_info=self.session_info,
                                         ssh_transport=self.control_session.get_transport(),
                                         sessions_rootdir=self.sessions_rootdir,
                                         session_instance=self.session_instance,
                                         proxy_options=self.proxy_options,
-                                        logger=self.logger
+                                        logger=self.logger,
                                        )
         self.proxy_subprocess, proxy_ok = self.proxy.start_proxy()
 
@@ -1731,7 +1796,7 @@ class X2GoTerminalSession(object):
             if self.params.kbtype not in ('null/null', 'auto') and (self.params.kblayout not in ('null', '') or self.params.kbvariant not in ('null', '')):
                 self.set_keyboard(layout=self.params.kblayout, variant=self.params.kbvariant)
 
-            if self.params.session_type in ('D', 'S'):
+            if self.get_session_type() in ('D', 'S'):
                 self.find_session_window()
                 self.auto_session_window_title()
                 self.raise_session_window()
@@ -1826,6 +1891,20 @@ class X2GoTerminalSession(object):
 
             self._cleaned_up = True
 
+    def is_kdrive_session(self):
+        """\
+        Test if this terminal session is a KDrive based desktop session.
+
+
+        :returns: ``True`` if this session is of session type KDrive ('K').
+
+        :rtype: ``bool``
+
+        """
+        if not self.session_info.is_initialized():
+            self.params.rewrite_session_type()
+        return self.get_session_type() == 'K'
+
     def is_rootless_session(self):
         """\
         Test if this terminal session is a rootless session.
@@ -1836,8 +1915,9 @@ class X2GoTerminalSession(object):
         :rtype: ``bool``
 
         """
-        self.params.rewrite_session_type()
-        return self.params.session_type == 'R'
+        if not self.session_info.is_initialized():
+            self.params.rewrite_session_type()
+        return self.get_session_type() == 'R'
 
     def is_shadow_session(self):
         """\
@@ -1849,8 +1929,9 @@ class X2GoTerminalSession(object):
         :rtype: ``bool``
 
         """
-        self.params.rewrite_session_type()
-        return self.params.session_type == 'S'
+        if not self.session_info.is_initialized():
+            self.params.rewrite_session_type()
+        return self.get_session_type() == 'S'
 
     def is_pubapp_session(self):
         """\
@@ -1862,6 +1943,6 @@ class X2GoTerminalSession(object):
         :rtype: ``bool``
 
         """
-        self.params.rewrite_session_type()
-        return self.params.session_type == 'P'
-
+        if not self.session_info.is_initialized():
+            self.params.rewrite_session_type()
+        return self.get_session_type() == 'P'
