@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (C) 2010-2018 by Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
+# Copyright (C) 2010-2020 by Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
 #
 # Python X2Go is free software; you can redistribute it and/or modify
 # it under the terms of the GNU Affero General Public License as published by
@@ -27,7 +27,6 @@
 X2GoProcessIniFile - helper class for parsing .ini files
 
 """
-from __future__ import print_function
 __NAME__ = 'x2goinifiles-pylib'
 
 __package__ = 'x2go'
@@ -35,6 +34,7 @@ __name__    = 'x2go.inifiles'
 
 # modules
 import os
+import sys
 try:
     import configparser
 except:
@@ -218,13 +218,16 @@ class X2GoIniFile(object):
         """
         if self.user_config_file and self._write_user_config:
             try:
-                fd = open(self.user_config_file, 'wb')
+                if sys.version_info[0] >= 3:
+                    fd = open(self.user_config_file.encode(), 'w')
+                else:
+                    fd = open(self.user_config_file.encode(), 'wb')
                 self.iniConfig.write(fd)
                 fd.close()
                 self._write_user_config = False
                 return True
             except Exception as e:
-                print(e)
+                self.logger('failure during write operation of %s; reported error: %s' % (self.user_config_file, e), loglevel=log.loglevel_ERROR, )
         return False
     __write = write
 
